/*-*- mode:c;indent-tabs-mode:nil;c-basic-offset:2;tab-width:8;coding:utf-8 -*-│
│vi: set net ft=c ts=2 sts=2 sw=2 fenc=utf-8                                :vi│
╞══════════════════════════════════════════════════════════════════════════════╡
│ Copyright 2020 Justine Alexandra Roberts Tunney                              │
│                                                                              │
│ Permission to use, copy, modify, and/or distribute this software for         │
│ any purpose with or without fee is hereby granted, provided that the         │
│ above copyright notice and this permission notice appear in all copies.      │
│                                                                              │
│ THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL                │
│ WARRANTIES WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED                │
│ WARRANTIES OF MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE             │
│ AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL         │
│ DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR        │
│ PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER               │
│ TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR             │
│ PERFORMANCE OF THIS SOFTWARE.                                                │
╚─────────────────────────────────────────────────────────────────────────────*/
#include "libc/calls/calls.h"
#include "libc/calls/internal.h"
#include "libc/calls/struct/stat.internal.h"
#include "libc/dce.h"
#include "libc/intrin/asan.internal.h"
#include "libc/intrin/describeflags.internal.h"
#include "libc/intrin/strace.internal.h"
#include "libc/intrin/weaken.h"
#include "libc/sysv/errfuns.h"
#include "libc/zipos/zipos.internal.h"

/**
 * Returns information about file, via open()'d descriptor.
 *
 * @return 0 on success or -1 w/ errno
 * @raise EBADF if `fd` isn't a valid file descriptor
 * @raise EIO if an i/o error happens while reading from file system
 * @raise EOVERFLOW shouldn't be possible on 64-bit systems
 * @asyncsignalsafe
 */
int fstat(int fd, struct stat *st) {
  int rc;
  if (__isfdkind(fd, kFdZip)) {
    rc = _weaken(__zipos_fstat)(
        (struct ZiposHandle *)(intptr_t)g_fds.p[fd].handle, st);
  } else if (!IsWindows() && !IsMetal()) {
    rc = sys_fstat(fd, st);
  } else if (IsMetal()) {
    rc = sys_fstat_metal(fd, st);
  } else if (!__isfdkind(fd, kFdFile)) {
    rc = ebadf();
  } else {
    rc = sys_fstat_nt(__getfdhandleactual(fd), st);
  }
  STRACE("fstat(%d, [%s]) → %d% m", fd, DescribeStat(rc, st), rc);
  return rc;
}
