/*-*- mode:c;indent-tabs-mode:nil;c-basic-offset:2;tab-width:8;coding:utf-8 -*-│
│vi: set net ft=c ts=2 sts=2 sw=2 fenc=utf-8                                :vi│
╞══════════════════════════════════════════════════════════════════════════════╡
│ Copyright 2023 Justine Alexandra Roberts Tunney                              │
│                                                                              │
│ Permission to use, copy, modify, and/or distribute this software for         │
│ any purpose with or without fee is hereby granted, provided that the         │
│ above copyright notice and this permission notice appear in all copies.      │
│                                                                              │
│ THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL                │
│ WARRANTIES WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED                │
│ WARRANTIES OF MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE             │
│ AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL         │
│ DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR        │
│ PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER               │
│ TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR             │
│ PERFORMANCE OF THIS SOFTWARE.                                                │
╚─────────────────────────────────────────────────────────────────────────────*/
#include "libc/calls/syscall-sysv.internal.h"
#include "libc/dce.h"
#include "libc/intrin/strace.internal.h"
#include "libc/macros.internal.h"
#include "libc/nexgen32e/rdtsc.h"
#include "libc/runtime/internal.h"
#include "libc/runtime/memtrack.internal.h"
#include "libc/runtime/runtime.h"
#include "libc/runtime/stack.h"
#include "libc/runtime/syslib.internal.h"
#include "libc/sysv/consts/prot.h"
#include "libc/thread/thread.h"
#include "libc/thread/tls.h"
#ifndef __x86_64__

void __wipe(uintptr_t);
int main(int, char **, char **) __attribute__((__weak__));

typedef int init_f(int argc, char **argv, char **envp, unsigned long *auxv);

extern long syscon_start[];
extern long syscon_end[];
extern long syscon_linux[];
extern long syscon_xnu[];
extern long syscon_freebsd[];
extern long syscon_openbsd[];
extern long syscon_netbsd[];
extern long syscon_windows[];
extern init_f __strace_init;
extern init_f *__preinit_array_start[] __attribute__((__weak__));
extern init_f *__preinit_array_end[] __attribute__((__weak__));
extern init_f *__init_array_start[] __attribute__((__weak__));
extern init_f *__init_array_end[] __attribute__((__weak__));
extern char ape_stack_vaddr[] __attribute__((__weak__));
extern char ape_stack_memsz[] __attribute__((__weak__));
extern char ape_stack_prot[] __attribute__((__weak__));
extern pthread_mutex_t __mmi_lock_obj;
extern int hostos asm("__hostos");

textstartup void cosmo(long *sp, struct Syslib *m1) {
  int argc;
  init_f **fp;
  uintptr_t *pp;
  long *mp, *magnums;
  char **argv, **envp;
  unsigned long *auxv;

  // get startup timestamp as early as possible
  // its used by --strace and also kprintf() %T
  kStartTsc = rdtsc();

  // extracts arguments from old sysv stack abi
  argc = *sp;
  argv = (char **)(sp + 1);
  envp = (char **)(sp + 1 + argc + 1);
  auxv = (unsigned long *)(sp + 1 + argc + 1);
  while (*auxv++) donothing;

  // detect apple m1 environment
  if ((__syslib = m1)) {
    hostos = _HOSTXNU;
    magnums = syscon_xnu;
  } else {
    hostos = _HOSTLINUX;
    magnums = syscon_linux;
  }

  // setup system magic numbers
  for (mp = syscon_start; mp < syscon_end; ++mp) {
    *mp = *magnums++;
  }

  // needed by kisdangerous()
  __oldstack = (intptr_t)sp;
  __pid = sys_getpid().ax;

  // initialize memory manager
  _mmi.n = ARRAYLEN(_mmi.s);
  _mmi.p = _mmi.s;
  __mmi_lock_obj._type = PTHREAD_MUTEX_RECURSIVE;

  // record system-provided stack to memory manager
  _mmi.i = 1;
  _mmi.p->x = (uintptr_t)GetStackAddr() >> 16;
  _mmi.p->y = (uintptr_t)(GetStackAddr() + (GetStackSize() - FRAMESIZE)) >> 16;
  _mmi.p->size = GetStackSize();
  _mmi.p->prot = PROT_READ | PROT_WRITE;

#if 0
#if IsAsan()
  __asan_init(argc, argv, envp, auxv);
#endif
#endif

  InitializeFileDescriptors();

  // set helpful globals
  __argc = argc;
  __argv = argv;
  __envp = envp;
  __auxv = auxv;
  environ = envp;
  if (argc) program_invocation_name = argv[0];

  // run initialization callbacks
  _init();
  __enable_tls();
#ifdef SYSDEBUG
  argc = __strace_init(argc, argv, envp, auxv);
#endif
  for (fp = __init_array_end; fp-- > __init_array_start;) {
    (*fp)(argc, argv, envp, auxv);
  }

  // run program
  if (!IsTiny()) __wipe(0);
  exit(main(argc, argv, envp));
}

#endif /* __x86_64__ */
